unit ucalibrador;

{$mode delphi}

interface

uses
  Classes, SysUtils, Math,
  uparques_modeloempirico,
  uparques_modeloteorico,
  uauxiliares,
  xmatdefs, fddp, umodelosintcegh,ucaminos_modelado_ro,
  matreal, ucomplex, uro_aire;

const
  ReductorEntropia = 0.7;
  UmbralCNT_Dividir = 10;
  MaxCNT_Div_Entropia = 10;

  // Cantidad mínima de muestars para que un cluster esté representado
    // en el modelo empírico.

type

  { TCalibradorParque }

  TCalibradorParque = class

    parque_ME: TParque_ModeloEmpirico;
    parque_MT: TParque_ModeloTeorico;


    // punteros a los parámetros dentro del parque
    paramsPtr: TDAOfNRealPtr;

    // parametros de la iteración de búsqueda del óptimo
    // mejor punto evaluado _xo y propuesto como siguiente _xs
    params_xo, params_xs: TDAOfNReal;

    // Valor de la función objetivo en el mejor punto y en el propuesto como siguiente
    fo, fs: NReal;
    entropia: NReal;



    constructor Create( parque: TParque_ModeloTeorico; archi_series: string);
    constructor CreateLoadFromArchi(archi: string);
    procedure StoreInArchi(archi: string);

    // Recorre los sectores y tramos del modelo empírico
    // y calcula el error cuadrático entre la potencia asociada
    // a dicho secto y tramo en el modelo empírico y la potencia
    // calculada con el modelo teórico para la potencia media de viento
    // asociada al mismo punto del modelo empírico.
    function Costo(params: TDAOfNReal; semilla: integer): NReal;

    // Realiza la calibración del juego de parámetros cuyos ordinales se pasan
    // en el array parsidx. Si parsidx = nil se realiza la calibración sobre
    // todos los parámetros.
    function Calibrar(parsidx: TDAOfNInt): NREal;

    procedure GetParams(prms: TDAOfNreal);
    procedure SetParams(prms: TDAOfNreal);
    procedure Calc_xs(parsidx: TDAOfNInt);


    // Procedimiento auxiliar para verificación de coherencia
    // del modelo empirico con la curva del fabricante.
    // Escribe archivo de texto con salida del modelo empírico para
    // velocidades entre 0 y 20 m/s de a 0.5 m/s
    procedure WriteMamtrizComparativaCurvas(archi: string);

    // Escribe mapa de potencias calculadas durante la llamada a Costo.
    procedure writeMapas( MapaPot_ME, MapaPot_MT: TMatR );


    procedure Free;

    constructor CreateLoadFromFile(var f: textfile);
    procedure StoreInFile(var f: textfile);
  end;


implementation


procedure TCalibradorParque.Free;
begin
//  if parque_ME <> nil then parque_ME.Free;
//  if parque_MT <> nil then parque_MT.Free;
  setlength( paramsPtr, 0 );
  setlength( params_xo, 0 );
  setlength( params_xs, 0 );
  inherited Free;
end;


procedure TCalibradorParque.GetParams(prms: TDAOfNreal);
var
  k: integer;
begin
  for k := 0 to high(prms) do
    prms[k] := paramsPtr[k]^;
end;

procedure TCalibradorParque.SetParams(prms: TDAOfNreal);
var
  k: integer;
begin
  for k := 0 to high(prms) do
    paramsPtr[k]^ := prms[k];
end;

procedure TCalibradorParque.Calc_xs(parsidx: TDAOfNInt);
var
  k: integer;
begin
  if parsidx = nil then
  begin
    for k := 0 to high(params_xs) do
    begin
      params_xs[k] := max(params_xo[k] + (2 * (random - 0.5) * 0.1) * entropia, 0);
    end;
  end
  else
  begin
    for k := 0 to high(params_xs) do
      params_xs[k] := params_xo[k];

    for k := 0 to high(parsidx) do
      params_xs[parsidx[k]] :=
        max(params_xo[parsidx[k]] + (2 * (random - 0.5) * 0.1) * entropia, 0);
  end;
end;

constructor TCalibradorParque.Create(parque: TParque_ModeloTeorico; archi_series: string);
begin
  inherited Create;
  parque_ME:= TParque_ModeloEmpirico.Create( parque, archi_series );
  parque_MT:= parque;
  self.paramsPtr := parque_MT.ParametrosPtr;
  setlength(params_xo, length(paramsPtr));
  setlength(params_xs, length(paramsPtr));
  GetParams(params_xo);
end;


constructor TCalibradorParque.CreateLoadFromFile( var f: textfile );
begin
  inherited Create;
  parque_MT := TParque_ModeloTeorico.Create_LoadFromFile(f);
  parque_ME := TParque_ModeloEmpirico.Create_LoadFromFile(f, parque_MT );
  paramsPtr := parque_MT.ParametrosPtr;
  setlength(params_xo, length(paramsPtr));
  setlength(params_xs, length(paramsPtr));
  GetParams(params_xo);
 end;

constructor TCalibradorParque.CreateLoadFromArchi(archi: string);
var
  f: textfile;
begin
  setSeparadoresGlobales;
  assignfile(f, archi);
  reset(f);
  CreateLoadFromFile( f );
  closefile(f);
  setSeparadoresLocales;
end;

procedure TCalibradorParque.StoreInFile( var f: textfile );
begin
  Parque_MT.StoreInFile(f);
  Parque_ME.StoreInFile(f);
end;
procedure TCalibradorParque.StoreInArchi(archi: string);
var
  f: textfile;
begin
  setSeparadoresGlobales;
  assignfile(f, archi);
  rewrite(f);
  StoreInFile( f );
  closefile(f);
  setSeparadoresLocales;
end;



procedure TCalibradorParque.WriteMamtrizComparativaCurvas(archi: string);
var
  f: textfile;
  kang: integer;
  ang: NReal;
  ir, pe: NReal;
  pv3: NReal;
  kvel: integer;
  peso: Nreal;
begin
  assignfile(f, archi);
  rewrite(f);

  for kvel := 0 to 40 do
    Write(f, #9, (kvel / 2): 12: 3);
  writeln(f);

  for kang := 1 to parque_ME.NSectores do
  begin
    ang :=parque_ME.AngOfjSector( kang );
    Write(f, ang);
    for kvel := 0 to 40 do
    begin
      pv3 := pv3Ofv( kvel / 2, ro_BASE);
      pe := parque_ME.Calc_PE(ang, pv3, ro_BASE, peso);
      Write(f, #9, pe);
    end;
    writeln(f);
  end;


  Write(f, 'A0');
  for kvel := 0 to 40 do
  begin
    pv3 := pv3Ofv(kvel / 2, ro_BASE);
    ir := parque_MT.Molinos[0].curva.curva_pv3.inv_interpol(pv3);
    pe := parque_MT.Molinos[0].curva.curva_P.interpol(ir) * length(parque_MT.Molinos);
    Write(f, #9, pe);
  end;
  writeln(f);

  closefile(f);
end;


procedure TCalibradorParque.writeMapas( MapaPot_ME, MapaPot_MT: TMatR );
var
  sal: textfile;
  kAng, kTramo: integer;
begin
  assignfile( sal, 'c:\basura\mapas.xlt' );
  rewrite( sal );

  for kTramo:= 1 to MapaPot_ME.nc do
    write( sal, #9, kTramo );
  writeln( sal  );

  writeln( sal  );
  writeln( sal , ' Potencia según modelo empírico.' );
  for kTramo:= 1 to MapaPot_ME.nc do
    write( sal, #9, vOfpv3( parque_ME.Representantes_pv3_medio.e(kTramo), ro_BASE ) );
  writeln( sal  );

  for kAng:= 1 to MapaPot_ME.nf do
  begin
    write(sal, parque_ME.AngOfjSector( kAng ) );
    for kTramo:= 1 to MapaPot_ME.nc do
      write( sal, #9, MapaPot_ME.e(kang, kTramo ) );
    writeln( sal  );

  end;

  writeln( sal  );
  writeln( sal , ' Potencia según modelo teórico.' );
  for kTramo:= 1 to MapaPot_ME.nc do
    write( sal, #9, vOfpv3( parque_ME.Representantes_pv3_medio.e(kTramo), ro_BASE ) );
  writeln( sal  );

  for kAng:= 1 to MapaPot_ME.nf do
  begin
    write(sal, parque_ME.AngOfjSector( kAng ) );
    for kTramo:= 1 to MapaPot_ME.nc do
      write( sal, #9, MapaPot_MT.e(kang, kTramo ) );
    writeln( sal  );

  end;

  writeln( sal  );
  writeln( sal , ' Potencia según modelo Empírico - Teórico.' );
  for kTramo:= 1 to MapaPot_ME.nc do
    write( sal, #9, vOfpv3( parque_ME.Representantes_pv3_medio.e(kTramo), ro_BASE ) );
  writeln( sal  );

  for kAng:= 1 to MapaPot_ME.nf do
  begin
    write(sal, parque_ME.AngOfjSector( kAng ) );
    for kTramo:= 1 to MapaPot_ME.nc do
      write( sal, #9, MapaPot_ME.e(kang, kTramo ) - MapaPot_MT.e(kang, kTramo ) );
    writeln( sal  );

   end;
  closefile( sal );
end;



function TCalibradorParque.Costo(params: TDAOfNReal; semilla: integer ): NReal;
var
  err2_acum, err2: NReal;
   PE_MW_Modelo, PE_MW_Medida: NReal;
  ang: NReal;
  pv3Anemo, pv3Anemo_s: NReal;
  convergio: boolean;

  pv3_Libre: NReal;

  kCronica, kVel: integer;
  NAngulos, NVels: integer;
  fuente:  TMadreUniforme;
begin
  SetParams(params);
  // cálculo del objetivo
  err2_acum := 0;

  fuente:=  TMadreUniforme.Create( semilla );
  NAngulos:= 5;
  NVels:= 5;
  for kCronica:= 1 to NAngulos  do
  begin
    ang:= fuente.rnd * 360;
    parque_MT.Calc_PotMW_init_(ang);
    for kVel := 1 to NVels do
    begin
      pv3Anemo :=  pv3Ofv( 3 + fuente.rnd * (15-3 ), ro_BASE );
      PE_MW_Modelo := parque_MT.Calc_PotMW_anemo( ro_BASE, pv3Anemo,
      parque_MT.Potencia_Autorizada_MW, convergio, pv3Anemo_s, pv3_Libre );

      PE_MW_Medida := parque_ME.Calc_PE_curva_s( ang, pv3Anemo, ro_BASE );

      err2 := sqr(PE_MW_Medida - PE_MW_Modelo) ;
      err2_acum := err2_acum + err2; // * PE_MW_Medida / 100.0;
    end;
  end;

  fuente.Free;
  Result := err2_acum;
end;


function TCalibradorParque.Calibrar(parsidx: TDAOfNInt): NREal;
var
  cnt_iter: integer;
  cnt_fracasos: integer;
  cnt_exitos: integer;
  itersufix: string;
  j: integer;

begin
  randomize;
  fo := Costo(params_xo, 31 );
  entropia := 1;
  cnt_iter := 0;
  cnt_fracasos := 0;
  cnt_exitos := 0;

  IterSufix := '';
  for j := 0 to high(parsidx) do
    IterSufix := IterSufix + IntToHex(parsidx[j], 2);

  while cnt_iter < MaxCNT_Div_Entropia do
  begin
    writeln(cnt_iter: 6, ', E:', entropia: 6: 4, ', fo: ', fo: 6: 4,
      ', cntfs: ', cnt_fracasos: 6);
    Calc_xs(parsidx);
    fs := Costo(params_xs, 31);
    if fs < fo then
    begin
      cnt_fracasos := 0;
      vswap(params_xo, params_xs);
      fo := fs;
      Inc(cnt_exitos);

      StoreInArchi('c:\basura\calibrado_' + parque_MT.Nombre + IterSufix +
        IntToStr(cnt_exitos) + '.clp');
    end
    else
      Inc(cnt_fracasos);

    if cnt_fracasos >= UmbralCNT_Dividir then
    begin
      entropia := entropia * ReductorEntropia;
      cnt_fracasos := 0;
      Inc(cnt_iter);
    end;
  end;

  SetParams(params_xo);
  Result := fo;

end;




end.
