unit uArcos;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  Classes,
  SysUtils, xmatdefs, usimplex, uglobs,
  uNodos,
  uActorNodal,
  ucosa,
  uCosaConNombre,
  ufechas, uconstantesSimSEE, uranddispos,
  uFichasLPD,
  uFuentesAleatorias,
  uConversions;

resourcestring
  rsArcoSimple = 'Arco';

type

  { TFichaArco }

  TFichaArco = class(TFichaLPD)
  public
    (**************************************************************************)
    (*               A T R I B U T O S   P E R S I S T E N T E S              *)
    (**************************************************************************)

    rendimiento: TDAofNReal;
    peaje: TDAofNReal;
    PMax: TDAofNreal;
    fd: NReal; // factor de disponibilidad
    tRepHoras: NReal;
    flg_ConsiderarPeajeEnElDespacho: boolean;
    flg_SumarPeajeAlCDP: boolean;
    factorPeajeCDP: NReal;
    PagoPorDisponibilidad_USD_MWh: NReal; // [USD/MWh] Pago por Potencia
    (**************************************************************************)


    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      rendimiento, peaje, PMax: TDAofNReal; fd: NReal; tRepHoras: NReal;
      ConsidearrPeajeEnElDespacho, SumarPeajeAlCDP: boolean;
      FactorPeajeCDP: NReal; PagoPorDisponibilidad_USD_MWh: NReal);
    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}
    function InfoAd_: string; override;
    procedure Free; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
    class function CreateDataConversionList: TListDataConversion; override;
    {$ENDIF}
    published
      {$IFDEF MIGRACION_PERSISTENCIA}
      property _rendimiento: TDAofNReal                  read rendimiento                     write rendimiento;
      property _peaje: TDAofNReal                        read peaje                           write peaje;
      property _PMax: TDAofNreal                         read PMax                            write PMax;
      property _fd: NReal                                read fd                              write fd;
      property _tRepHoras: NReal                         read tRepHoras                       write tRepHoras;
      property _flg_ConsiderarPeajeEnElDespacho: boolean read flg_ConsiderarPeajeEnElDespacho write flg_ConsiderarPeajeEnElDespacho;
      property _flg_SumarPeajeAlCDP: boolean             read flg_SumarPeajeAlCDP             write flg_SumarPeajeAlCDP;
      property _factorPeajeCDP: NReal                    read factorPeajeCDP                  write factorPeajeCDP;
      property _PagoPorDisponibilidad_USD_MWh: NReal     read PagoPorDisponibilidad_USD_MWh   write PagoPorDisponibilidad_USD_MWh;
      {$ENDIF}
  end;



  { TArco }

  TArco = class(TActorBiNodal)
  public
    NLineasDisponibles: integer;
    PMaxDisponible_: TDAofNReal;
    // resultados del despacho
    P_Entrante: TDAOfNReal; //[MW] Potencia Entrante

    //Costo por poste de la transmisión
    //costo[iposte]:= P[iposte] * pa.peaje * globs.durpos[iposte];
    costo: TDAofNReal;
    // valor por poste del multiplicador de Lagrange de la restrición de PMax
    costo_congestion: TDAOfNReal; //[USD/MW]

    pa: TFichaArco;
    Condicion_Fuente: TFuenteAleatoria;
    Condicion_nombreBorne: string;
    Condicion_numeroBorne: integer;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades, lpd: TFichasLPD; Entrada, Salida: TNodo;
  Condicion_Fuente: TFuenteAleatoria; Condicion_NombreBorne: string);

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure RegistrarParametrosDinamicos; override;
    function InfoAd_: string; override;
    class function DescClase: string; override;

    procedure SorteosDelPaso(sortear: boolean); override;
    procedure prepararPaso_ps; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;
    procedure opt_cargue(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_leerSolucion(s: TSimplex); override;

    function getNombreVar(ivar: integer; var nombre: string): boolean; override;
    function getNombreRes(ires: integer; var nombre: string): boolean; override;

    procedure dump_Variables(var f: TextFile; charIndentacion: char); override;

    procedure PubliVars; override;
    procedure Free; override;

    function get_pa_FD(kTipoUnidad: integer): NReal; override;
    function get_pa_TMR(kTipoUnidad: integer): NReal; override;

    procedure AfterInstantiation; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
     class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; override;
     class function CreateDataConversionList(): TListDataConversion; override;
     {$ENDIF}
   published
     {$IFDEF MIGRACION_PERSISTENCIA}
     property _Condicion_Fuente:                 TFuenteAleatoria  read fuente                 write fuente;
     property _Condicion_nombreBorne:            string            read nombreBorne            write nombreBorne;
     {$ENDIF}
  end;



procedure AlInicio;
procedure AlFinal;

implementation

uses uActores;


//-------------------
// Métodos de TArco
//===================

constructor TArco.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades, lpd: TFichasLPD; Entrada, Salida: TNodo;
  Condicion_Fuente: TFuenteAleatoria;
  Condicion_NombreBorne: string );
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, Entrada, Salida);
  self.lpd := lpd;
  self.Condicion_Fuente := Condicion_Fuente;
  self.Condicion_NombreBorne := Condicion_NombreBorne;
  self.Condicion_numeroBorne:= -1;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TArco.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('lpd', TCosa(lpd));

  if f.Version >= 139 then
  begin
    f.rdReferencia('Condicion_fuente', TCosa(Condicion_fuente), Self);
    f.rd('Condicion_nombreBorne', Condicion_nombreBorne);
    Condicion_numeroBorne:= -1;
  end
  else
  begin
    Condicion_Fuente:= nil;
    Condicion_NombreBorne:= '';
    Condicion_numeroBorne:= -1;
  end;
  f.EjecutarLectura;
  lpd.Propietario := self;

end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TArco.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('lpd', lpd);
  f.wrReferencia('Condicion_fuente', Condicion_fuente);
  f.wr('Condicion_nombreBorne', Condicion_nombreBorne);
end;
{$ENDIF}

procedure TArco.PrepararMemoria(globs: TGlobs);
var
  n, k, j: integer;
  af: TFichaArco;

begin
  inherited prepararMemoria(globs);
  setlength(P_Entrante, globs.NPostes);
  SetLength(costo, globs.NPostes);
  setlength(costo_congestion, globs.NPostes);
  setlength(PMaxDisponible_, globs.NPostes);

  if Condicion_fuente <> nil then
    Condicion_numeroBorne := Condicion_fuente.idBorne(Condicion_nombreBorne);


  for j := 0 to self.lpd.Count - 1 do
  begin
    af := self.lpd.items[j] as TFichaArco;
    if (length(af.rendimiento) <> globs.NPostes) then
    begin
      n := length(af.rendimiento);
      setlength(af.rendimiento, globs.NPostes);
      setlength(af.peaje, globs.NPostes);
      setlength(af.PMax, globs.NPostes);
      for k := n to high(af.rendimiento) do
        af.rendimiento[k] := af.rendimiento[n - 1];
      for k := n to high(af.peaje) do
        af.peaje[k] := af.peaje[n - 1];
      for k := n to high(af.PMax) do
        af.PMax[k] := af.PMax[n - 1];
    end;
  end;

end;

procedure TArco.RegistrarParametrosDinamicos;
begin
  inherited registrarParametrosDinamicos;
  lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pA, nil);
end;

function TArco.InfoAd_: string;
begin
  Result := '';
end;

class function TArco.DescClase: string;
begin
  Result := rsArcoSimple;
end;


function TArco.get_pa_FD(kTipoUnidad: integer): NReal;
begin
  Result := pa.fd;
end;

function TArco.get_pa_TMR(kTipoUnidad: integer): NReal;
begin
  Result := pa.tRepHoras;
end;

procedure TArco.AfterInstantiation;
begin
  inherited AfterInstantiation;
  lpd.Propietario := self;
end;

procedure TArco.SorteosDelPaso(sortear: boolean);
var
  kPoste: integer;
begin
  if globs.ObligarDisponibilidad_1_ then
  begin
    NLineasDisponibles := paUnidades.nUnidades_Instaladas[0]-paUnidades.nUnidades_EnMantenimiento[0];
    for kPoste := 0 to globs.NPostes - 1 do
      PMaxDisponible_[kPoste] := pa.PMax[kPoste] * NLineasDisponibles;
  end
  else if sortear then
  begin
    ActualizarProbabilidadesReparacionYRotura_(pa.fd, pa.tRepHoras);
    NLineasDisponibles := Sorteos_RepRotUnidades;
    for kPoste := 0 to globs.NPostes - 1 do
      PMaxDisponible_[kPoste] := pa.PMax[kPoste] * NLineasDisponibles;
  end
  else
  begin
    NLineasDisponibles := paUnidades.nUnidades_Instaladas[0]-paUnidades.nUnidades_EnMantenimiento[0];
    for kPoste := 0 to globs.NPostes - 1 do
      PMaxDisponible_[kPoste] := pa.PMax[kPoste] * pa.fd * NLineasDisponibles;
  end;
end;

procedure TArco.prepararPaso_ps;
begin
  if (Condicion_Fuente<> nil) and ( Condicion_Fuente.Bornera[ Condicion_numeroBorne ] <= 0) then
  begin
    Vclear( PMaxDisponible_ );
    NLineasDisponibles:= 0;
  end
end;

procedure TArco.opt_nvers(var ivar, ivae, ires: integer);
begin
  Self.ivar := ivar;
  if NLineasDisponibles > 0 then
    ivar := ivar + globs.NPostes;
end;

procedure TArco.opt_cargue(s: TSimplex);
var
  ibaseResSal, ibaseResEnt: integer;
  iposte: integer;
begin
  if NLineasDisponibles > 0 then
  begin
{$IFDEF SPXCONLOG}
    for iposte := 0 to globs.NPostes - 1 do
      s.set_NombreVar(ivar + iposte, Nombre + '_P[MW]' + IntToStr(iposte + 1));
{$ENDIF}
    // Tenemos que aportar a las restricciones de
    // demanda de los nodos de
    ibaseResEnt := NodoA.ires;
    ibaseResSal := NodoB.ires;
    for iposte := 0 to globs.NPostes - 1 do
    begin
      s.pon_e(ibaseResEnt + iposte, ivar + iposte, -1);
      s.pon_e(ibaseResSal + iposte, ivar + iposte, pa.rendimiento[iposte]);

      // Ahora agregamos a la función de UTILIDAD (-costo)
      if pa.flg_ConsiderarPeajeEnElDespacho then
        s.pon_e(s.nf, ivar + iposte, -pa.peaje[iposte] * globs.durpos[iposte]);
    end;
  end;
end;

procedure TArco.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  iposte: integer;
begin
  if NLineasDisponibles > 0 then
    // Le fijamos como cota máxima PMax a la potencia en todos los postes
    for iposte := 0 to globs.NPostes - 1 do
      s.cota_sup_set(ivar + iposte, PMaxDisponible_[iposte]);
end;

procedure TArco.opt_leerSolucion(s: TSimplex);
var
  iposte: integer;
  m: NReal;
  P: NReal;
begin
  CostoDirectoDelPaso := 0;
  Ingreso_PorDisponibilidad := 0;
  Ingreso_PorEnergia := 0;
  vclear(P_Entrante);
  vclear(P_NodoA);
  vclear(P_NodoB);
  vclear(costo_congestion);

  if NLineasDisponibles > 0 then
  begin
    if pa.PagoPorDisponibilidad_USD_MWh <> 0 then
      for iposte := 0 to globs.NPostes - 1 do
        Ingreso_PorDisponibilidad :=
          Ingreso_PorDisponibilidad + pa.PagoPorDisponibilidad_USD_MWh *
          PMaxDisponible_[iposte] * globs.DurPos[iposte];

    // recuperamos los valores de Potencia despachada
    for iposte := 0 to globs.NPostes - 1 do
    begin
      P := s.xval(ivar + iposte);
      P_Entrante[iposte] := P;
      if pa.flg_SumarPeajeAlCDP then
      begin
        m := P * pa.peaje[iposte] * globs.DurPos[iposte];
        costo[iposte] := m;
        Ingreso_PorEnergia := Ingreso_PorEnergia + m;
      end;
      costo_congestion[iposte] := -s.xmult(ivar + iposte);
      P_NodoA[iposte] := -P;
      P_NodoB[iposte] := P * pa.rendimiento[iposte];
    end;

    costoDirectoDelPaso := Ingreso_PorDisponibilidad;
    if pa.flg_SumarPeajeAlCDP then
      costoDirectoDelPaso := costoDirectoDelPaso + Ingreso_PorEnergia *
        pa.factorPeajeCDP;
  end;
end;

function TArco.getNombreVar(ivar: integer; var nombre: string): boolean;
begin
  if (ivar >= self.ivar) and (ivar < self.ivar + globs.NPostes) then
  begin
    nombre := self.Nombre + '_P[MW]' + IntToStr(ivar - self.ivar + 1);
    Result := True;
  end
  else
    Result := False;
end;

function TArco.getNombreRes(ires: integer; var nombre: string): boolean;
begin
  Result := False;
end;

procedure TArco.dump_Variables(var f: TextFile; charIndentacion: char);
var
  iposte: integer;

begin
  inherited dump_Variables(f, charIndentacion);

  Write(f, charIndentacion, 'peaje=');
  for iposte := 0 to high(pa.peaje) do
  begin
    if iposte > 0 then
      Write(f, ',');
    Write(f, ' ' + FloatToStrF(pa.peaje[iposte], ffFixed, 10, 3));
  end;
  writeln(f);

  Write(f, charIndentacion, 'PMax=');
  for iposte := 0 to high(pa.PMax) do
  begin
    if iposte > 0 then
      Write(f, ',');
    Write(f, ' ', FloatToStrF(pa.PMax[iposte], ffFixed, 10, 3));
  end;
  writeln(f);

  writeln(f);
end;

procedure TArco.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('P', '[MW]', 6, 1, P_Entrante, True, True);
  PublicarVariableVR('Costo', '[USD]', 6, 1, costo, True, True);
  PublicarVariableVR('CostoCongestion', '[USD/MW]', 6, 1, costo_congestion, True, True);
  PublicarVariableNI('NLineasDisponibles', '[u]', NLineasDisponibles, True);
end;

procedure TArco.Free;
begin
  if lpd <> nil then
    lpd.Free;
  setlength(P_Entrante, 0);
  SetLength(costo, 0);
  SetLength(costo_congestion, 0);
  setlength(PMaxDisponible_, 0);
  inherited Free;
end;

//---------------------
//Metodos de TFichaArco
//=====================

constructor TFichaArco.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; rendimiento, peaje, PMax: TDAofNReal;
  fd: NReal; tRepHoras: NReal; ConsidearrPeajeEnElDespacho, SumarPeajeAlCDP: boolean;
  FactorPeajeCDP: NReal; PagoPorDisponibilidad_USD_MWh: NReal);
begin
  inherited Create(capa, fecha, periodicidad);
  self.rendimiento := rendimiento;
  self.peaje := peaje;
  self.PMax := PMax;
  self.fd := fd;
  self.tRepHoras := tRepHoras;
  self.flg_ConsiderarPeajeEnElDespacho := ConsidearrPeajeEnElDespacho;
  self.flg_SumarPeajeAlCDP := SumarPeajeAlCDP;
  self.factorPeajeCDP := FactorPeajeCDP;
  self.PagoPorDisponibilidad_USD_MWh := PagoPorDisponibilidad_USD_MWh;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TFichaArco.Create_ReadFromText(f: TArchiTexto);
var
  rendimiento_aux, peajeaux, PMaxaux: NReal;
begin
  flg_ConsiderarPeajeEnElDespacho := True;
  flg_SumarPeajeAlCDP := True;
  factorPeajeCDP := 1.0;

  if f.Version < 14 then
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('rendimiento', rendimiento_aux);
    f.rd('peaje', peajeaux);
    f.rd('fd', fd);
    f.rd('PMax', PMaxaux);
    f.EjecutarLectura;
    self.tRepHoras := 8;
  end
  else if f.Version < 69 then
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('rendimiento', rendimiento_aux);
    f.rd('peaje', peajeaux);
    f.rd('fd', fd);
    f.rd('tRepHoras', tRepHoras);
    f.rd('PMax', PMaxaux);
    f.EjecutarLectura;
  end
  else
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('rendimiento', rendimiento);
    f.rd('peaje', peaje);
    f.rd('PMax', PMax);
    f.rd('fd', fd);
    f.rd('tRepHoras', tRepHoras);
    if f.Version >= 70 then
    begin
      f.rd('ConsiderarPeajeEnElDespacho', flg_ConsiderarPeajeEnElDespacho);
      f.rd('SumarPeajeAlCDP', flg_SumarPeajeAlCDP);
      f.rd('factorPeajeCDP', factorPeajeCDP);
    end;
    f.EjecutarLectura;
  end;

  if f.Version < 69 then
  begin
    // hago esto y luego en preparar memoria vemos
    // si no conincide la cantidad de postes lo extiendo.
    setlength(rendimiento, 1);
    setlength(peaje, 1);
    setlength(PMax, 1);
    rendimiento[0] := rendimiento_aux;
    peaje[0] := peajeaux;
    PMax[0] := PMaxaux;
  end;

  if f.Version < 98 then
    PagoPorDisponibilidad_USD_MWh := 0
  else
    f.rd('PagoPorDisponibilidad_USD_MWh', PagoPorDisponibilidad_USD_MWh);

end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TFichaArco.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('rendimiento', rendimiento);
  f.wr('peaje', peaje);
  f.wr('PMax', PMax);
  f.wr('fd', fd);
  f.wr('tRepHoras', tRepHoras);

  f.wr('ConsiderarPeajeEnElDespacho', flg_ConsiderarPeajeEnElDespacho);
  f.wr('SumarPeajeAlCDP', flg_SumarPeajeAlCDP);
  f.wr('factorPeajeCDP', factorPeajeCDP);

  f.wr('PagoPorDisponibilidad_USD_MWh', PagoPorDisponibilidad_USD_MWh);
end;
{$ENDIF}


function TFichaArco.InfoAd_: string;
begin
  Result := 'PMáx= ' + DAOfNRealToStr_(PMax, 10, 1, ';') + ' MW, ' +
    'fDisp= ' + FloatToStrF(fd, ffGeneral, 10, 2) + ' p.u., ' +
    'tRep= ' + FloatToStrF(tRepHoras, ffGeneral, 10, 1) + 'h, ' +
    'Rendimiento= ' + DAOfNRealToStr_(rendimiento, 10, 2, ';') +
    ' p.u., ' + 'peaje= ' + DAOfNRealToStr_(peaje, 10, 1, ';') + ' USD/MWh';
end;

procedure TFichaArco.Free;
begin
  setlength(rendimiento, 0);
  setlength(peaje, 0);
  setlength(PMax, 0);

  inherited Free;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TFichaArco.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);

  Result.AddDoubleColumn('rendimiento', 0, 0, 69);
  Result.AddDAOfDoubleColumn('rendimiento', 69);

  Result.AddDoubleColumn('peaje', 0, 0, 69);
  Result.AddDAOfDoubleColumn('peaje', 69);

  Result.AddDoubleColumn('PMax', 0, 0, 69);
  Result.AddDAOfDoubleColumn('PMax', 69);

  Result.AddDoubleColumn('tRepHoras', 8, 0, 69);
  Result.AddBooleanColumn('flg_ConsiderarPeajeEnElDespacho', True, 70);
  Result.AddBooleanColumn('flg_SumarPeajeAlCDP', True, 70);
  Result.AddDoubleColumn('factorPeajeCDP', 1.0, 70);
  Result.AddDoubleColumn('PagoPorDisponibilidad_USD_MWh', 0, 98);

end;

class function TFichaArco.CreateDataConversionList: TListDataConversion;
begin

  Result:=inherited CreateDataConversionList;
  Result.Add(69, ['rendimiento', 'peaje', 'PMax'], ['rendimiento', 'peaje', 'PMax'],
    Result.ConversionPrimitiveToDynArray());

end;
{$ENDIF}

procedure AlInicio;
begin
  registrarClaseDeCosa(TArco.ClassName, TArco);
  registrarClaseDeCosa(TFichaArco.ClassName, TFichaArco);
end;

procedure AlFinal;
begin
end;

end.
